#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Device import Device
from Homevee.Item.Person import Person
from Homevee.Utils.Database import Database

class PersonManager():
    def __init__(self):
        return

    def add_edit_person(self, user, id, name, nickname, address, latitude, longitude, phonenumber, birthdate, db: Database = None):
        if db is None:
            db = Database()
        try:
            person = Device.load_from_db(Person, id, db)
            person.name = name
            person.nickname = nickname
            person.phone_number = phonenumber
            person.address = address
            person.birthdate = birthdate
            person.longitude = longitude
            person.latitude = latitude
        except:
            person = Person(name, nickname, phonenumber, address, birthdate, longitude, latitude)

        return person.api_save()

    def get_persons(self, db):
        persons = Person.load_all(db)
        return {'persons': Person.list_to_dict(persons)}

    def delete_person(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        person = Device.load_from_db(Person, id)
        return person.api_delete()


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime
import json

from Homevee.DeviceAPI.heating import get_thermostat_info
from Homevee.Functions.nutrition_data import get_user_nutrition_overview
from Homevee.Functions.sensor_data import get_sensor_data
from Homevee.Functions.weather import get_weather
from Homevee.Helper import Logger
from Homevee.Item.Room import Room
from Homevee.Manager.CalendarManager import CalendarManager
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *

STATIC_DASHBOARD_ITEMS = ['calendar', 'weather', 'tvprogramme', 'drivingtime', 'nutritionmanager']

class DashboardManager():
    def __init__(self):
        return

    def get_user_dashboard(self, user, db: Database = None):
        if db is None:
            db = Database()
        user_dashboard = user.dashboard_data

        if user_dashboard is None:
            data = {'static': [], 'dynamic': []}
        else:
            data = json.loads(user_dashboard)

        dashboard_data = {}
        dashboard_data['static'] = data['static']

        if dashboard_data is not None and dashboard_data != "":

            dashboard_keys = []
            for item in dashboard_data['static']:
                Logger.log(item)
                dashboard_keys.append(item['id'])


            for item in STATIC_DASHBOARD_ITEMS:
                if item not in dashboard_keys:
                    Logger.log(item+" not existing")
                    dashboard_data['static'].append({'id': item, 'checked': False})

            dashboard_data['dynamic'] = []

            '''
            dynamic_temp_items = data['dynamic']
            devices_map = {}
            dynamic_items = []
    
            print dynamic_temp_items
    
            for item in dynamic_temp_items:
                found = False
    
                if(item['type'] not in devices_map):
                    devices_map[item['type']] = []
    
                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True
    
                if not found:
                    dynamic_items.append(item)
    
            print dynamic_items
            '''

            devices_map = {}
            for item in data['dynamic']:
                found = False

                if (item['type'] not in devices_map):
                    devices_map[item['type']] = []

                if item['id'] not in devices_map[item['type']]:
                    devices_map[item['type']].append(item['id'])
                else:
                    found = True

                if not found:
                    dashboard_data['dynamic'].append(self.get_device_info(user, item['type'], item['id']))

            return dashboard_data

        return {'result': 'unknownuser'}

    def get_user_dashboard_items(self, user, db: Database = None):
        if db is None:
            db = Database()
        dashboard_array = []

        data = user.dashboard_data

        if data is None:
            return {'dashboard': dashboard_array}

        data = json.loads(data)

        static_items = data['static']

        static_data = {}

        for item in static_items:
            static_data[item['id']] = item['checked']

        dynamic_temp_items = data['dynamic']
        devices_map = {}
        dynamic_items = []

        Logger.log(dynamic_temp_items)

        for item in dynamic_temp_items:
            found = False

            if (item['type'] not in devices_map):
                devices_map[item['type']] = []

            if item['id'] not in devices_map[item['type']]:
                devices_map[item['type']].append(item['id'])
            else:
                found = True

            if not found:
                dynamic_items.append(item)

        Logger.log(dynamic_items)

        # Wetter hinzufügen
        if 'weather' in static_data:
            if static_data['weather']:
                weather_data = get_weather(1)
                dashboard_array.append({'id': 'weather', 'data': weather_data[0]})

        # TV Programm hinzufügen
        if 'tvprogramme' in static_data:
            if static_data['tvprogramme']:
                dashboard_array.append({'id': 'tvprogramme', 'data': None})

        # Kalender hinzufügen
        if 'calendar' in static_data:
            if static_data['calendar']:
                calendar_entries = CalendarManager().get_calendar_day_items(user, datetime.datetime.now().strftime("%Y-%m-%d"))

                items = calendar_entries['calendar_entries']

                data = None
                for item in items:
                    start = datetime.datetime.strptime(item['start'], '%H:%M')
                    dnow = datetime.datetime.now()

                    if (dnow.time() > start.time()):
                        continue

                    if data is None:
                        data = item['name'] + " um " + item['start']
                    else:
                        data += '\n' + item['name'] + " um " + item['start'] + " Uhr"

                if data is None:
                    data = 'Keine Termine heute'

                dashboard_array.append({'id': 'calendar', 'data': data})

        # Fahrzeit
        if 'drivingtime' in static_data:
            if static_data['drivingtime']:
                dashboard_array.append({'id': 'drivingtime', 'data': '27 min'})

        # Ernährungsmanager
        try:
            if 'nutritionmanager' in static_data:
                if static_data['nutritionmanager']:
                    data_item = get_user_nutrition_overview(user)

                    if 'nutrition_day_data' in data_item['data']:
                        calories_left = data_item['data']['nutrition_day_data']['calories_left']
                        if (calories_left >= 0):
                            text = str(calories_left) + ' Kalorien übrig'
                        else:
                            text = str(calories_left) + ' Kalorien über deinem Ziel'

                        dashboard_array.append({'id': 'nutritionmanager', 'data': text})
                    else:
                        dashboard_array.append({'id': 'nutritionmanager',
                                                'data': "Ernährungsmanager noch nicht eingerichtet"})
        except:
            pass

        if len(dynamic_items) > 0:
            dashboard_array.append({'id': 'favouritedevices', 'data': None})

        return {'dashboard': dashboard_array}

    def edit_user_dashboard(self, username, dashboard_data, db: Database = None):
        if db is None:
            db = Database()


        dashboard_data = json.loads(dashboard_data)

        #Check duplicates

        dynamic_temp_items = dashboard_data['dynamic']
        devices_map = {}
        dynamic_items = []

        Logger.log(dynamic_temp_items)

        for item in dynamic_temp_items:
            found = False

            if (item['type'] not in devices_map):
                devices_map[item['type']] = []

            if item['id'] not in devices_map[item['type']]:
                devices_map[item['type']].append(item['id'])
            else:
                found = True

            if not found:
                dynamic_items.append(item)

        Logger.log(dynamic_items)

        dashboard_data['dynamic'] = dynamic_items

        #Check duplicates end

        dashboard_data_json = json.dumps(dashboard_data)

        db.update("UPDATE USERDATA SET DASHBOARD_DATA = :data WHERE USERNAME == :username",
                    {'data': dashboard_data_json, 'username': username})

        return self.get_user_dashboard(username)

    def get_user_favourite_devices(self, user, db: Database = None):
        if db is None:
            db = Database()
        data = json.loads(user.dashboard_data)

        dynamic_items = data['dynamic']

        Logger.log("data: "+str(data))

        fav_array = self.get_dynamic_items(user, dynamic_items)

        return fav_array

    def get_dynamic_items(self, user, dynamic_items, db: Database = None):
        if db is None:
            db = Database()
        rooms = Room.load_all(db)

        favourites = {}

        room_map = {}

        for room in rooms:
            if(user.has_permission(room.id)):
                favourites[int(room.id)] = []
                room_map[room.id] = room.get_dict()
            else:
                continue

        types = {}

        for item in dynamic_items:
            #print item
            if item['type'] not in types:
                types[item['type']] = []

            types[item['type']].append(item['id'])
            Logger.log(types)

        for type in types:
            ids = types[type]
            if len(ids) > 0:
                for id in ids:
                    item = self.get_device_info(user, type, id)

                    if item is None:
                        continue

                    Logger.log(favourites)

                    favourites[int(item['room'])].append(item)

        output = []
        for location in list(favourites.keys()):
            room_item = room_map[location]

            if len(favourites[location]) > 0:
                room_item['devices'] = favourites[location]
                output.append(room_item)

        return output

    def get_device_info(self, user, type, id, db: Database = None):
        if db is None:
            db = Database()
        Logger.log(type + " - " + id)

        device_data = None

        try:
            if type == FUNKSTECKDOSE:
                item = db.select_one("SELECT * FROM FUNKSTECKDOSEN WHERE DEVICE == :id", {'id': id})
                device_data = {'room': item['ROOM'], 'name': item['NAME'], 'icon': item['ICON'],
                               'value': item['ZUSTAND'] == "1",
                               'type': 'switch', 'devicetype': type, 'id': id}
            elif type == PHILIPS_HUE_LIGHT:
                item = db.select_one("SELECT * FROM PHILIPS_HUE_LIGHTS WHERE ID == :id", {'id': id})
                device_data = {'room': item['LOCATION'], 'name': item['NAME'],
                               'icon': item['ICON'],
                               'value': {'is_on': False, 'brightness': 100, 'color': item['HUE']},
                               'type': 'rgb', 'devicetype': type, 'id': id}
            #URL-Geräte
            elif type == URL_RGB_LIGHT:
                item = db.select_one("SELECT * FROM URL_RGB_LIGHT WHERE ID = :id", {'id': id})
                device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'], 'type': 'rgb',
                               'icon': item['ICON'], 'value': {'is_on': False, 'brightness': 100, 'color': item['LAST_COLOR']}}
            elif type == URL_TOGGLE:
                item = db.select_one("SELECT * FROM URL_TOGGLE WHERE ID = :id",{'id': id})
                device_data = {'name': item['NAME'], 'room': item['LOCATION'], 'id': item['ID'],
                                    'icon': item['ICON'], 'value': '', 'type': 'toggle'}
            #Z-Wave
            elif type == ZWAVE_THERMOSTAT:
                item = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE THERMOSTAT_ID == :id", {'id': id})
                data = get_thermostat_info(user, item['RAUM'], type, item['THERMOSTAT_ID'])
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['RAUM'], 'type': 'heating',
                        'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
            elif type == ZWAVE_DIMMER:
                item = db.select_one("SELECT * FROM ZWAVE_DIMMER WHERE ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            elif type == ZWAVE_POWER_METER:
                item = db.select_one("SELECT * FROM ZWAVE_THERMOSTATS WHERE DEVICE_ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['DEVICE_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            elif type == ZWAVE_SENSOR:
                item = db.select_one("SELECT * FROM ZWAVE_SENSOREN WHERE ID == :id", {'id': id})
                device_data = {'name': item['SHORTFORM'], 'id': item['ID'],
                        'icon': item['ICON'], 'room': item['RAUM'], 'type': 'value',
                               'value': get_sensor_data(user, item['RAUM'], type, id, 1)}
            elif type == ZWAVE_SWITCH:
                item = db.select_one("SELECT * FROM ZWAVE_SWITCHES WHERE THERMOSTAT_ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['THERMOSTAT_ID'],
                        'icon': item['ICON'], 'room': item['LOCATION'], 'type': 'heating',
                        'value': item['VALUE']}
            #MAX Cube
            elif type == MAX_THERMOSTAT:
                item = db.select_one("SELECT * FROM MAX_THERMOSTATS WHERE ID == :id", {'id': id})
                data = get_thermostat_info(user, item['RAUM'], type, id)
                device_data = {'name': item['NAME'], 'id': item['ID'], 'room': item['RAUM'], 'type': 'heating',
                        'icon': item['ICON'],
                        'value': data['value'], 'min': data['min'], 'max': data['max'], 'step': 0.5}
            #Wake on Lan
            elif type == WAKE_ON_LAN:
                item = db.select_one("SELECT * FROM WAKE_ON_LAN WHERE DEVICE == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['DEVICE'], 'icon': item['ICON'], 'room': item['LOCATION'],
                               'value': '', 'type': 'wakeonlan'}
            elif type == XBOX_ONE_WOL:
                item = db.select_one("SELECT * FROM XBOX_ONE_WOL WHERE ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['ID'], 'icon': item['ICON'], 'room': item['LOCATION'],
                               'value': '', 'type': 'wakeonlan'}

            #MQTT Sensor
            elif type == MQTT_SENSOR:
                item = db.select_one("SELECT * FROM MQTT_SENSORS WHERE ID == :id", {'id': id})
                device_data = {'name': item['NAME'], 'id': item['ID'],
                               'icon': item['ICON'], 'room': item['ROOM'], 'type': 'value',
                               'value': get_sensor_data(user, item['ROOM'], type, id, 1)}
        except:
            Logger.log("Error loading device")

        if device_data is None or not user.has_permission(device_data['room']):
            return None

        device_data['roomname'] = Room.get_name_by_id(device_data['room'])
        device_data['devicetype'] = type

        return device_data

        '''//DIY
            case "DIY Gerät":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_DEVICES' WHERE DEVICE == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['ROOM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => "", "type" => "button");
                break;
            case "DIY Sensor":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_SENSORS' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getSensorData($query['RAUM'], $type, $id, "1", $db), "type" => "value");
            case "DIY Fensterkontakt":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_REEDSENSORS' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "reed");
                break;
            case "DIY Anwesenheitssensor":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_PRESENCESENSORS' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getReedData($query['RAUM'], $type, $id, $db), "type" => "presence");
                break;
            case "DIY Schalter":
                $dbQuery = $db->prepare("SELECT * FROM 'DIY_SWITCHES' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                $devicedata = array("room" => $query['RAUM'], "name" => $query['NAME'], "icon" => $query['ICON'],
                "value" => getModes($query['RAUM'], $type, $id, $db) === "1", "type" => "switch");
                break;
            case "IP Kamera":
                $dbQuery = $db->prepare("SELECT * FROM 'IP_CAMERA' WHERE ID == :id");
                $dbQuery->execute($queryParams);
                
                $query = $dbQuery->fetch(PDO::FETCH_ASSOC);
                
                $ip = $query['IP'];
                $port = $query['PORT'];
                    
                if($port != null && $port != ""){
                    $port = ":$port";
                }
                    
                $path = $query['PATH'];
                    
                $url = "http://$ip$port$path";
                
                $devicedata = array('room' => $query['LOCATION'], 'name' => $query['NAME'], 'icon' => $query['ICON'],
                'value' => "", 'url' => $url, 'username' => $query['USERNAME'], 'password' => $query['PASSWORD'],
                'type' => "ipcamera");
                break;
                
            default:
                return null;
        }
        
        if(!hasPermission($devicedata['room'], $db) || $devicedata['room'] == null){
            return null;
        }
        else{
            $devicedata['roomname'] = $roomArray[$devicedata['room']];
            $devicedata['devicetype'] = $type;
            $devicedata['id'] = $id;
            
            return $devicedata;
        }
    }
    ?>
        '''



import json
import os
import urllib
from _thread import start_new_thread

import pip
from packaging import version

from Homevee.Item import User
from Homevee.Item.Status import *
from Homevee.Utils import Constants
from Homevee.Utils.Database import Database
from Homevee.Utils.NotificationManager import NotificationManager


class Updater:
    def __init__(self):
        return

    def get_homevee_update_version(self):
        """
        Gets the updateable homevee-version
        :return: None if no newer version found, version number otherwise
        """
        installed_version = Constants.HOMEVEE_VERSION_NUMBER

        newest_version = self.get_newest_version()

        if(newest_version is None):
            return None

        if(version.parse(newest_version) > version.parse(installed_version)):
            return newest_version
        else:
            return None

    def get_newest_version(self) -> str:
        """
        Gets the newest homevee-version from pypi
        :return: the version string or None
        """
        url = "https://pypi.org/pypi/Homevee/json"

        try:
            response = urllib.request.urlopen(url).read()
            response = response.decode('utf-8')
            response_json = json.loads(response)
            version = response_json['info']['version']

            return version
        except:
            return None

    def check_for_updates(self) -> dict:
        """
        Checks if a new homevee-version is available on pypi
        :return:
        """
        new_version = self.get_homevee_update_version()

        return {
            'updates':{
                'current_version': Constants.HOMEVEE_VERSION_NUMBER,
                'new_version': new_version,
                'update_available': (new_version is not None),
                'changelog': "Changelog blabla..." #TODO add changelog or link to actual changelog
            }
        }

    def do_homevee_update(self, user: User, db: Database) -> dict:
        """
        Starts the homevee update-process
        :param user: the user
        :param db: the database connection
        :return: the status of the update process
        """
        if(not user.has_permission("admin")):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        start_new_thread(self.update_thread, ())

        return Status(type=STATUS_OK).get_dict()

    def update_thread(self):
        """
        The thread for the update process of homevee
        :return:
        """
        new_version = self.get_homevee_update_version()

        try:
            pip.main(["install", "--upgrade", "Homevee"])
        except:
            return False

        #Datenbank upgraden
        Database().upgrade()

        # TODO texte lokalisieren
        title = "Update"
        body = "Update auf Version " + new_version

        # Send notification to admin
        NotificationManager().send_notification_to_admin(title, body, Database())

        # Reboot the system after the update
        os.system('reboot')


from Homevee.Item.Room import Room
from Homevee.Utils.Database import Database

class SmartSpeakerManager():
    def __init__(self):
        return

    def get_smart_speakers(self, user, db: Database = None):
        if db is None:
            db = Database()
        speakers =[]

        results = db.select_all("SELECT * FROM SMART_SPEAKER ORDER BY LOCATION ASC", {})

        for speaker in results:
            if user.has_permission(speaker['LOCATION']):
                item = {'name': speaker['NAME'], 'id': speaker['ID'], 'key': speaker['KEY'],
                        'location': speaker['LOCATION'], 'location_name': Room.get_name_by_id(speaker['LOCATION'],
                                                                                              db)}

                speakers.append(item)

        return {'speakers': speakers}

    def add_edit_smart_speaker(self, user, db: Database = None):
        if db is None:
            db = Database()
        return


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.DeviceAPI.blind_control import get_blinds
from Homevee.DeviceAPI.dimmer import get_dimmers
from Homevee.DeviceAPI.energy_data import get_power_usage_devices
from Homevee.DeviceAPI.get_modes import get_modes
from Homevee.DeviceAPI.heating import get_thermostats
from Homevee.DeviceAPI.rgb_control import get_rgb_devices
from Homevee.DeviceAPI.toggle_devices import get_toggle_devices
from Homevee.DeviceAPI.wake_on_lan import get_xbox_devices, get_wol_devices
from Homevee.Functions.sensor_data import get_sensor_data
from Homevee.Functions.triggers import get_triggers
from Homevee.Helper import Logger
from Homevee.Item import Item
from Homevee.Item.Room import Room
from Homevee.Item.Status import *
from Homevee.Item.User import User
from Homevee.Manager.SceneManager import SceneManager
from Homevee.Utils.Database import Database
from Homevee.Utils.DeviceTypes import *

CONTROL_TYPE_SWITCH = "switch"
CONTROL_TYPE_VALUE = "value"
CONTROL_TYPE_DIMMER = "dimmer"
CONTROL_TYPE_TOOGLE = "toggle"
CONTROL_TYPE_TRIGGER = "trigger"
CONTROL_TYPE_RGB = "rgb"
CONTROL_TYPE_HEATING = "heating"
CONTROL_TYPE_SCENES = "scenes"
CONTROL_TYPE_WOL = "wwakeonlan"
CONTROL_TYPE_XBOXONE = "xboxone"
CONTROL_TYPE_BLINDS = "blinds"



class RoomDataManager():
    def __init__(self):
        return

    def get_rooms(self, user: User, db: Database = None) -> list:
        if db is None:
            db = Database()
        rooms = []
        all_rooms = Room.load_all(db)
        for room in all_rooms:
            if(user.has_permission(room.id)):
                rooms.append(room)
        return Room.list_to_dict(rooms)

    def get_room_data(self, user: User, room, db: Database = None) -> dict:
        if db is None:
            db = Database()
        if not user.has_permission(room):
            return Status(type=STATUS_NO_PERMISSION).get_dict()

        roomdata = []

        room = Item.load_from_db(Room, room, db)

        #Schalter abfragen
        switchdata = get_modes(user, room, "", "")
        for item in switchdata['modi']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['device_type'],
                'icon': item['icon'], 'type': CONTROL_TYPE_SWITCH, 'value': item['mode']})

        #Dimmer
        dimmerdata = get_dimmers(user, room)
        for item in dimmerdata['dimmer']:
            roomdata.append({'name': item['name'], 'id': item['device'], 'devicetype': item['type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_DIMMER, 'value': item['value']})

        #Toggle
        toggledata = get_toggle_devices(user, room)
        for item in toggledata['toggles']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['type'],
                'icon': item['icon'], 'type': CONTROL_TYPE_TOOGLE, 'value': ''})

        #Trigger
        triggerdata = get_triggers(user, room)
        for item in triggerdata['triggers']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype':item['type'],
                'icon': item['icon'], 'type': CONTROL_TYPE_TRIGGER, 'value': ''})

        #RGB
        rgbdata = get_rgb_devices(user, room)
        for item in rgbdata['rgb']:
            Logger.log(item)
            roomdata.append({'name': item['name'], 'devicetype':item['type'], 'id': item['id'],
                'icon': item['icon'], 'type': CONTROL_TYPE_RGB, 'value': item['value']})

        #Thermostate
        thermostat_data = get_thermostats(user, room)
        for item in thermostat_data['thermostats'][0]['thermostat_array']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_HEATING, 'value': item['data']['value'],
                             'step': 0.5, 'min': item['data']['min'], 'max': item['data']['max']})

        #Sensoren abfragen
        sensordata = get_sensor_data(user, room)
        for item in sensordata['values'][0]['value_array']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['device_type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_VALUE, 'value': item['value']})

        #Szenen abfragen
        scenedata = SceneManager().get_scenes(user, room)
        if len(scenedata['scenes']) > 0:
            roomdata.append({'name': "Szenen", 'id': "scenes", 'devicetype': "scenes",
                             'icon': "scenes", 'type': CONTROL_TYPE_SCENES, 'value': ""})

        #WOL abfragen
        wol_devices = get_wol_devices(user, room)
        for item in wol_devices['devices']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': WAKE_ON_LAN,
                             'icon': item['icon'], 'type': CONTROL_TYPE_WOL})

        #XBOX ONE WOL abfragen
        xbox_devices = get_xbox_devices(user, room)
        for item in xbox_devices['devices']:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': XBOX_ONE_WOL,
                             'icon': item['icon'], 'type': CONTROL_TYPE_XBOXONE})

        #Jalousie-Steuerung
        blind_controls = get_blinds(user, room)['blinds']
        for item in blind_controls:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['type'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_BLINDS, 'value': item['value']})

        #Strom-Messgeräte
        power_usage_devices = get_power_usage_devices(user, room)
        for item in power_usage_devices:
            roomdata.append({'name': item['name'], 'id': item['id'], 'devicetype': item['devicetype'],
                             'icon': item['icon'], 'type': CONTROL_TYPE_VALUE, 'value': item['value']})

        return {'roomdata': roomdata}


#!/usr/bin/python
# -*- coding: utf-8 -*-
import datetime

from Homevee.Functions.nutrition_data import get_user_fitness_profile
from Homevee.Helper import Logger
from Homevee.Utils.Database import Database
from Homevee.VoiceAssistant.Helper import generate_string
from Homevee.VoiceAssistant.Modules import VoiceModule

MAX_PORTION_DEVIATION = 10

class VoiceGetNutritionDiaryModule(VoiceModule):
    def get_pattern(self, db: Database = None):
        if db is None:
            db = Database()
        return [
            ['wie', ['viel', 'viele'], ['fett', 'kalorien', 'kohlenhydrate', 'eiweiß', 'protein', 'zucker'], ['ist', 'sind', 'habe'], ['übrig', 'offen', 'auf', 'erlaubt']],
            ['wie', ['viel', 'viele'], ['fett', 'kalorien', 'kohlenhydrate', 'eiweiß', 'protein', 'zucker'], ['darf', 'kann', 'soll', 'muss'], ['essen', 'trinken', 'verzehren', ['zu', 'mir', 'nehmen']]]
        ]

    def get_label(self):
        return "nutritiondiary"

    def run_command(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        return self.voice_query_nutrition_diary(username, text, context, db)

    def voice_query_nutrition_diary(self, username, text, context, db: Database = None):
        if db is None:
            db = Database()
        words = text.split(" ")

        queried_value = None

        for word in words:
            if word == 'fett':
                queried_value = 'FAT'
            elif word == 'kalorien':
                queried_value = 'CALORIES'
            elif word == 'kohlenhydrate':
                queried_value = 'CARBS'
            elif word == 'zucker':
                queried_value = 'SUGAR'
            elif word in ['protein', 'proteine', 'eiweiß']:
                queried_value = 'PROTEIN'
            else:
                continue

            break

        user_profile = get_user_fitness_profile(username, db)

        Logger.log(user_profile)

        if (user_profile is False):
            answer = "Du hast noch kein Profil für den Ernährungsmanager erstellt. Das kannst du in der App im Menüpunkt Ernährungsmanager tun."
        elif queried_value is None:
            #Kein Wert gefunden
            answer = "Welchen Nährwert möchtest du abfragen?"

            answer_data = [
                ['Welcher Nährwert soll abgefragt werden.'],
                ['Welchen Nährwert ', ['magst', 'möchtest', 'willst'],' du abfragen.'],
            ]
        else:

            results = db.select_all("SELECT * FROM NUTRITION_DATA WHERE USER = :user AND DATE = :date",
                        {'user': username, 'date': datetime.datetime.today().strftime("%d.%m.%Y")})

            eaten_queried_value_today = 0

            for item in results:
                amount = (float(item['EATEN_PORTION_SIZE']) / float(item['PORTIONSIZE']))

                eaten_queried_value_today += item[queried_value] * amount

            unit = None

            if queried_value == 'FAT':
                value_left = user_profile['fatgoal']-eaten_queried_value_today
                unit = "Gramm Fett"
            elif queried_value == 'CALORIES':
                value_left = int(user_profile['caloriesgoal']-eaten_queried_value_today)
                eaten_queried_value_today = int(eaten_queried_value_today)
                unit = 'Kalorien'
            elif queried_value == 'CARBS':
                value_left = user_profile['carbsgoal']-eaten_queried_value_today
                unit = 'Gramm Kohlenhydrate'
            elif queried_value == 'SUGAR':
                value_left = user_profile['sugargoal']-eaten_queried_value_today
                unit = 'Gramm Zucker'
            elif queried_value == 'PROTEIN':
                value_left = user_profile['proteingoal']-eaten_queried_value_today
                unit = 'Gramm Protein'

            answer_data = [
                ['Du hast heute ', ['schon', 'bereits'], ' ', str(eaten_queried_value_today), " ", unit, ' ',
                 ['gegessen', 'verzehrt', 'aufgenommen', 'zu dir genommen'], ' und ', ['noch', 'weitere'], ' ',
                 str(value_left), ' ', unit, ' ', ['übrig', 'frei', 'offen'], '.']
            ]

            #answer = "Du hast heute schon "+str(eaten_queried_value_today)+" "+unit+" zu dir genommen und noch "+str(value_left)+" "+unit+" übrig."

        answer = generate_string(answer_data)

        return {'msg_speech': answer, 'msg_text': answer}


#!/usr/bin/python
# -*- coding: utf-8 -*-
from Homevee.Item.Status import *
from Homevee.Utils.Database import Database

class PlaceManager():
    def __init__(self):
        return

    def get_place(self, id, db: Database = None):
        if db is None:
            db = Database()
        if(id is None or id == "" or id is -1):
            return None

        place = db.select_one("SELECT * FROM PLACES WHERE ID = :id",
                    {'id': id})

        return place

    def get_my_places(self, user, db: Database = None):
        if db is None:
            db = Database()
        places = []

        results = db.select_all("SELECT * FROM PLACES", {})

        for place in results:
            places.append({'id': place['ID'], 'name': place['NAME'], 'address': place['ADDRESS'], 'latitude': place['LATITUDE'], 'longitude': place['LONGITUDE']})

        return {'places': places}

    def add_edit_place(self, user, id, name, address, latitude, longitude, db: Database = None):
        if db is None:
            db = Database()
        add_new = (id == None or id == "" or id == "-1")

        if (add_new):
            db.insert("INSERT INTO PLACES (NAME, ADDRESS, LATITUDE, LONGITUDE) VALUES (:name, :address, :latitude, :longitude)",
                        {'name': name, 'address': address, 'latitude': latitude, 'longitude': longitude})

            return Status(type=STATUS_OK).get_dict()

        else:
            db.update("UPDATE SCENES SET NAME = :name, ADDRESS = :address, LATITUDE = :latitude, LONGITUDE = :longitude WHERE ID = :id",
                        {'name': name, 'address': address, 'latitude': latitude, 'longitude': longitude, 'id': id})

            return Status(type=STATUS_OK).get_dict()

    def delete_place(self, user, id, db: Database = None):
        if db is None:
            db = Database()
        db.delete("DELETE FROM PLACES WHERE ID = :id", {'id': id})
        return Status(type=STATUS_OK).get_dict()


from abc import abstractmethod
from sqlite3 import Row
from typing import Type, List

from Homevee.Exception import NoPermissionException
from Homevee.Item import Item
from Homevee.Item.Status import Status, STATUS_ERROR, STATUS_OK
from Homevee.Item.User import User
from Homevee.Utils.Database import Database


class Manager():
    def __init__(self):
        return

    def api_add_edit(self, user: User, item: Type[Item], db: Database = None) -> Status:
        """
        Saves the given item and returns the status object of the transaction
        :param user: the user trying to do the action
        :param item: the item
        :param db: the database connection
        :return: the status
        """
        if not self.check_permission(user, item):
            raise NoPermissionException()
        
        if self.add_edit(item, db):
            return Status(type=STATUS_ERROR)
        else:
            return Status(type=STATUS_OK)
        
    def api_delete(self, user: User, item: Type[Item], db: Database = None) -> Status:
        """
        Deletes the given item and returns the status object of the transaction
        :param user: the user trying to do the action
        :param item: the item
        :param db: the database connection
        :return: the status
        """
        if not self.check_permission(user, item):
            raise NoPermissionException()
        
        if self.delete(item, db):
            return Status(type=STATUS_ERROR)
        else:
            return Status(type=STATUS_OK)

    def api_get_all(self, user: User, db: Database = None) -> list:
        """
        Gets all items (the user is allowed to manipulate) and returns them as a dict list
        :param user: the user trying to get all items
        :param db: the database connection
        :return: the list of dicts
        """
        all_items = self.get_all(db)

        output_items = []

        for item in all_items:
            if self.check_permission(user, item):
                output_items.append(item)

        return self.get_dict_list_from_item_list(all_items)

    def api_find_by_id(self, user: User, id: int, db: Database = None) -> dict:
        """
        Returns a single item identified by an id as a dict if the user is allowed to
        :param user: the user
        :param id: the id if the item to search
        :param db: the database connection
        :return: the dict of the item
        """
        item = self.find_by_id(id, db)

        if item is None:
            return None

        if self.check_permission(user, item):
            return None

        return self.get_dict_from_item(item)

    def add_edit(self, item: Type[Item], db: Database = None) -> bool:
        """
        Saves the given item and creates it if it does not exist yes
        :param item: the item
        :param db: the database connection
        :return: boolean if successful
        """
        if self.item_exists(item, db):
            return self.update(item, db)
        else:
            return self.create(item, db)

    def find_by_query(self, query: str, params: dict, db: Database = None) -> List[Type[Item]]:
        """
        Finds items identified by the given query
        :param query: the sql query
        :param params: the params-dict
        :param db: the database connection
        :return: the list of found items
        """
        if db is None:
            db = Database()

        items = []

        results = db.select_all(query, params)

        for result in results:
            items.append(self.create_item_from_db_result(result))

        return items

    def find_one_by_query(self, query: str, params: dict, db: Database = None) -> Type[Item]:
        """
        Finds a single item identified by the given query
        :param query: the sql query
        :param params: the params-dict
        :param db: the database connection
        :return: the found item
        """
        if db is None:
            db = Database()

        items = []

        results = db.select_all(query, params)

        for result in results:
            items.append(self.create_item_from_db_result(result))

        if len(items) > 0:
            return items[0]
        else:
            return None

    def get_dict_list_from_item_list(self, list: List[Type[Item]]) -> list:
        """
        Creates a dict list from a given list of items
        :param list: the list of dicts
        :return: the list of items
        """
        output = []
        for item in list:
            output.append(self.get_dict_from_item(item))
        return output

    def create_item_list_from_dict_list(self, list: list) -> List[Type[Item]]:
        """
        Creates a item list from a given list of dicts
        :param list: the list of items
        :return: the list of dicts
        """
        items = []
        for dict in list:
            items.append(self.construct_item_from_dict(dict))
        return items

    def get_dict_from_item(self, item: Type[Item]) -> dict:
        """
        Builds a dict from the given item
        :param item: the item
        :return: the built dict
        """
        return item.get_dict()

    @abstractmethod
    def create_item_from_db_result(self, result: Row) -> Type[Item]:
        """
        Creates an item from the database result row
        :param result: the database result row
        :return: the created item
        """
        raise NotImplementedError

    @abstractmethod
    def item_exists(self, item: Type[Item], db: Database) -> bool:
        """
        Checks whether the given item already exists or not
        :param item: the item
        :param db: the database connection
        :return: true if item exists, false if not
        """
        raise NotImplementedError

    @abstractmethod
    def create(self, item: Type[Item], db: Database) -> bool:
        """
        Creates the given item
        :param item: the item
        :param db: the database connection
        :return: boolean if successful
        """
        raise NotImplementedError

    @abstractmethod
    def update(self, item: Type[Item], db: Database) -> bool:
        """
        Updates the given item
        :param item: the item
        :param db: the database connection
        :return: boolean if successful
        """
        raise NotImplementedError

    @abstractmethod
    def delete(self, item: Type[Item], db: Database) -> bool:
        """
        Deletes the given item
        :param item: the item
        :param db: the database connection
        :return: boolean if successful
        """
        raise NotImplementedError

    @abstractmethod
    def get_all(self, db: Database) -> List[Type[Item]]:
        """
        Gets all existing items
        :return: the list of items
        """
        raise NotImplementedError

    @abstractmethod
    def find_by_id(self, id: int, db: Database) -> Type[Item]:
        """
        Finds a single item identified by the given id
        :param id: the id
        :param db: the database connection
        :return: the found item
        """
        raise NotImplementedError()

    @abstractmethod
    def construct_item_from_dict(self, dict: dict) -> Type[Item]:
        """
        Construct an item out of a given dict
        :param dict: the dict
        :return: the constructed item
        """
        raise NotImplementedError()

    @abstractmethod
    def check_permission(self, user: User, item: Type[Item]) -> bool:
        """
        Checks if the given user is allowed to manipulate the item
        :param user: the user
        :param item: the item
        :return: boolean if user is allowed
        """
        raise NotImplementedError


#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback
from sqlite3.dbapi2 import Row
from typing import Type, List

from Homevee.Exception import ItemNotFoundException
from Homevee.Item import Item
from Homevee.Item.Status import *
from Homevee.Item.User import User, Permission
from Homevee.Manager import Manager
from Homevee.Utils.Database import Database


class UserManager(Manager):
    def __init__(self):
        super(UserManager, self).__init__()
        return

    def item_exists(self, item: User, db: Database) -> bool:
        return self.find_by_id(item.id, db) is None

    def create(self, item: User, db: Database) -> bool:
        return db.insert("""INSERT INTO USERDATA (USERNAME, PASSWORD, IP, PERMISSIONS, PW_SALT) VALUES 
                                    (:username, :password, :ip, :permissions, :salt)""",
                  {'username': item.username, 'password': item.hashed_password, 'ip': item.ip,
                   'permissions': Permission.get_json_list(item.permissions), 'salt': item.salt}) != False

    def update(self, item: User, db: Database) -> bool:
        return db.update("""UPDATE USERDATA SET PASSWORD = :password, IP = :ip, PERMISSIONS = :permissions,
                                    PW_SALT = :salt WHERE ID = :id""",
                  {'password': item.hashed_password, 'ip': item.ip,
                   'permissions': Permission.get_json_list(item.permissions),
                   'salt': item.salt, 'id': item.id})

    def delete(self, item: User, db: Database) -> bool:
        return db.delete("DELETE FROM USERDATA WHERE ID == :id", {'user': item.id})

    def get_all(self, db: Database) -> List[Type[Item]]:
        return self.find_by_query('SELECT * FROM USERDATA', {}, db)

    def find_by_id(self, id: int, db: Database) -> Type[Item]:
        return self.find_one_by_query('SELECT * FROM USERDATA', {}, db)

    def find_by_usernames(self, usernames: list, db: Database) -> List[Type[Item]]:
        return self.find_by_query('SELECT * FROM USERDATA WHERE USERNAME IN (%s)' % ','.join('?'*len(usernames)),
                                     usernames, db)

    def construct_item_from_dict(self, dict: dict) -> Type[Item]:
        pass

    def check_permission(self, user: User, item: User) -> bool:
        return user.has_permission("admin") or (user.id == item.id)

    def create_item_from_db_result(self, result: Row) -> User:
        return User(result['USERNAME'], result['PASSWORD'], result['IP'], result['AT_HOME'],
                        result['EVENTS_LAST_CHECKED'], Permission.create_list_from_json(result['PERMISSIONS']),
                        result['DASHBOARD_DATA'], result['PW_SALT'], result['FCM_TOKEN'],
                        result['CURRENT_LOCATION'], result['ID'])


    def set_user_fcm_token(self, user, token, db: Database = None):
        if db is None:
            db = Database()
        user.fcm_token = token

        try:
            user.save_to_db(db)
            return Status(type=STATUS_OK).get_dict()
        except:
            return Status(type=STATUS_ERROR).get_dict()

    def has_users(self, db):
        users = User.load_all(db)
        return len(users) > 0

    def get_users(self, user, db: Database = None):
        if db is None:
            db = Database()
        if not user.hash_password("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        users = User.load_all(db)
        return {'userdata': User.list_to_dict(users)}

    def delete_user(self, user, user_to_delete, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            return Status(type=STATUS_NO_ADMIN).get_dict()

        user_to_delete = User.load_username_from_db(user_to_delete, db)
        return user_to_delete.api_delete()

    def add_edit_user(self, user, name, psw, ip, permissions, db: Database = None):
        if db is None:
            db = Database()
        if not user.has_permission("admin"):
            if user.username != name:
                return Status(type=STATUS_NO_PERMISSION).get_dict()

        hashed_pw, salt = User.hash_password(psw)

        try:
            edit_user = User.load_username_from_db(name, db)

            if not (psw == "" or psw is None):
                edit_user.hashed_password = hashed_pw
                edit_user.salt = salt

            edit_user.ip = ip
            edit_user.permissions = Permission.create_list_from_json(permissions)

        except ItemNotFoundException:
            traceback.print_exc()
            edit_user = User(username=name, hashed_password=hashed_pw, salt=salt, ip=ip,
                             permissions=Permission.create_list_from_json(permissions),
                             at_home=False, fcm_token=None, current_location=None,
                             dashboard_data=None, events_last_checked=0)

        print(edit_user.get_dict())

        return edit_user.api_save(db)